dofile('voxelfieldvisualize.lua')
dofile('noisemoduletree.lua')

-- Dimensions of the voxel field
VFX=64
VFY=64
VFZ=64


-- Some tweakable parameters

-- Semi-rare stone distribution
SEMIRARE_FREQUENCY = 2   -- Frequency of the semi-rare stone dist. Decrease to "spread out" deposits further across the map
SEMIRARE_DENSITY = 0.6   -- Increase this value to increase the density of semi-rare stone distribution

RARE_FREQUENCY = 3
RARE_GRADIENT_SCALE = 2
RARE_DENSITY = 0.4

DIRT_THRESHOLD = 0.08    -- Increase this value to thicken the top layer of dirt
BEDROCK_THRESHOLD = 0.95 -- Decrease this value to thicken the bottom layer of bedrock
GROUND_YSCALE = 0.1      -- Decrease this value toward 0 to increase the amount of turbulent "frothiness" of the ground-shape function.
                       -- A value of 0 forces the ground shape to act as a traditional height-map
CAVE_SIZE = 0.3          -- Increase this value to increase the relative "size" of cave tunnels
CAVE_TURBULENCE = 0.25


-- Module tree declaration

minecraftlevel2=
{
	{name="Open", type="constant", constant=1},
	{name="Dirt", type="constant", constant=2},
	{name="Stone", type="constant", constant=3},
	{name="SemiRare", type="constant", constant=4},
	{name="Rare", type="constant", constant=5},
	{name="Bedrock", type="constant", constant=6},
	
	{name="Constant1", type="constant", constant=1},
	{name="Constant0", type="constant", constant=0},
	
	{name="MainGradient", type="gradient", y1=0, y2=0.5},
	{name="MainGradientRemap", type="scaleoffset", source="MainGradient", scale=0.5, offset=0.5},
	
	{name="SemiRareFBM", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=4, frequency=SEMIRARE_FREQUENCY},
	{name="SemiRareFBMRemap", type="scaleoffset", source="SemiRareFBM", scale=0.5, offset=0.5},
	{name="SemiRareSelect", type="select", main_source="SemiRareFBMRemap", low_source="Stone", high_source="SemiRare", threshold=SEMIRARE_DENSITY, falloff=0},
	
	{name="RareFBM", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=3, frequency=RARE_FREQUENCY},
	{name="RareFBMRemap", type="scaleoffset", source="RareFBM", scale=0.5, offset=0.5},
	{name="RareFBMScale", type="scaleoffset", source="RareFBMRemap", scale=RARE_GRADIENT_SCALE, offset=0},
	{name="RareMult", type="combiner", combiner_type="MULTIPLY", source_0="RareFBMScale", source_1="MainGradientRemap"},
	{name="RareMultScale", type="scaleoffset", source="RareMult", scale=RARE_DENSITY, offset=0},
	{name="RareSelect", type="select", main_source="RareMultScale", low_source="SemiRareSelect", high_source="Rare", threshold=0.5, falloff=0},
	
	{name="DirtStoneSelect", type="select", main_source="MainGradientRemap", low_source="Dirt", high_source="RareSelect", threshold=DIRT_THRESHOLD, falloff=0},
	{name="GroundSelect", type="select", main_source="MainGradientRemap", low_source="Open", high_source="DirtStoneSelect", threshold=0.000001, falloff=0},

	
	{name="GroundShape", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=4, frequency=2},
	{name="GroundYScale", type="scaledomain", source="GroundShape", scaley=GROUND_YSCALE},
	{name="GroundTurb", type="turbulence", main_source="GroundSelect", y_axis_source="GroundYScale", y_power=0.5},
	
	{name="CaveShape1", type="fractal", fractal_type="RIDGEDMULTI", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=1, frequency=2},
	{name="CaveBase1", type="select", main_source="CaveShape1", low_source="Constant0", high_source="Constant1", threshold=1-CAVE_SIZE, falloff=0},
	
	{name="CaveShape2", type="fractal", fractal_type="RIDGEDMULTI", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=1, frequency=2, seed=1323},
	{name="CaveBase2", type="select", main_source="CaveShape2", low_source="Constant0", high_source="Constant1", threshold=1-CAVE_SIZE, falloff=0},
	{name="CaveMult", type="combiner", combiner_type="MULTIPLY", source_0="CaveBase1", source_1="CaveBase2"},
	
	{name="CaveTurbX", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=3, frequency=3, seed=1001},
	{name="CaveTurbY", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=3, frequency=3, seed=1201},
	{name="CaveTurbZ", type="fractal", fractal_type="FBM", basis_type="GRADIENT", interp_type="QUINTIC", num_octaves=3, frequency=3, seed=1301},
	{name="CaveTurb", type="turbulence", main_source="CaveMult", x_axis_source="CaveTurbX", y_axis_source="CaveTurbY", z_axis_source="CaveTurbZ", x_power=CAVE_TURBULENCE, y_power=CAVE_TURBULENCE, z_power=CAVE_TURBULENCE},
	
	
	{name="CaveInvert", type="scaleoffset", source="CaveTurb", scale=-1, offset=1},
	{name="CaveSelect", type="select", main_source="CaveInvert", low_source="Open", high_source="GroundTurb", threshold=0.5, falloff=0},
	
	{name="BedrockSelect", type="select", main_source="MainGradientRemap", low_source="CaveSelect", high_source="Bedrock", threshold=BEDROCK_THRESHOLD, falloff=0},
	
	{name="VisualizeRare", type="select", main_source="RareMultScale", low_source="Open", high_source="Rare", threshold=0.5, falloff=0},
	
}

-- Parse the module tree
mcl=NoiseModuleTree(minecraftlevel2)

-- Seed all modules
mcl:setSeedAll(randInt())
--mcl:setSeedAll(1245)

-- Set up a voxel field
vf=CVoxelField(VFX,VFY,VFZ)

-- Evaluate the module tree; BedrockSelect is the end=point.
-- The vector (0,0,0) is the starting point; modify to sample a different are of the function
-- Note that the function is designed so that the ground threshold is at Y=0.5, so keeping the origin Y component set to 0 is recommended;
-- use different X and Z values to sample different areas of the world.
vf:evaluateModule(mcl:getFunction("BedrockSelect"), 0,0,0)

-- Write the voxel field to a POVRay mesh file.
-- Parameters are filename, voxel field, dimensions of the voxel field, world-space offset at which to write the mesh, and the length
-- of the sides of an individual cube.
writeVoxelField("voxelfield.inc", vf, VFX, VFY, VFZ, {0,0,0}, 0.2)



