-- Voxel Field Visualization

-- Visualize voxel fields as meshes comprised as blocks.
-- Output a voxel field to a POVRay mesh using a table of blockID->Material mappings to determine how the blocks are textured

-- Table of ID->Material mappings
-- ID 0 denotes an out-of-bounds block, ie represents anything outside the voxel field
-- ID 1 denotes an empty space

-- Note that you need to add 1 before indexing the table, since Lua tables start indexing at 1

voxelfieldblocks=
{
	{texname="NONE", solid=false},
	{texname="NONE", solid=false},
	{texname="BLOCKTEX_1", solid=true },
	{texname="BLOCKTEX_2", solid=true },
	{texname="BLOCKTEX_3", solid=true },
	{texname="BLOCKTEX_4", solid=true },
	{texname="BLOCKTEX_5", solid=true }
}


-- Offsets for calculating the vertices of a given face of a block given the origin of the block
-- Top (x,y+1,z) - (x+1,y+1,z+1)
-- Bottom (x,y,z) - (x+1,y,z+1)
-- Left (x,y+1,z) - (x,y,z+1)
-- Right (x+1,y+1,z) - (x+1,y,z+1)
-- Front (x,y+1,z+1) - (x+1,y,z+1)
-- Back (x,y+1,z) - (x+1,y,z)

faceoffsets=
{
	{{0,1,0}, {1,1,0}, {0,1,1}, {1,1,1}},
	{{0,0,0}, {1,0,0}, {0,0,1}, {1,0,1}},
	{{0,1,0}, {0,1,1}, {0,0,0}, {0,0,1}},
	{{1,1,0}, {1,1,1}, {1,0,0}, {1,0,1}},
	{{0,1,1}, {1,1,1}, {0,0,1}, {1,0,1}},
	{{0,1,0}, {1,1,0}, {0,0,0}, {1,0,0}}
}

-- Table of offsets for calculating a block's neighbor based on a chosen face
-- Same order as above table: Top, Bottom, LEft, Right, Front Back
blockoffsets=
{
	{0,1,0},
	{0,-1,0},
	{-1,0,0},
	{1,0,0},
	{0,0,1},
	{0,0,-1}
}

function beginPOVMeshDeclaration(outfile)
	outfile:write("mesh {\n")
end

function closePOVMeshDeclaration(outfile)
	outfile:write("}\n")
end

function writePOVTriangleDeclaration(outfile, p1, p2, p3, texname)
	outfile:write("triangle {<"..p1[1]..","..p1[2]..","..p1[3]..">,<"..p2[1]..","..p2[2]..","..p2[3]..">,<"..p3[1]..","..p3[2]..","..p3[3].."> texture {"..texname.."} }\n")
end

function writeBlockFace(outfile, p1, p2, p3, p4, texname)
	writePOVTriangleDeclaration(outfile,p1,p3,p2,texname)
	writePOVTriangleDeclaration(outfile,p2,p3,p4,texname)
end



function testIsSolid(voxelfield, p)
	local v=voxelfield:getVoxel(p[1], p[2], p[3])+1
	if voxelfieldblocks[v]==nil then
		-- Block type not supported, default to false
		return false
	end
	return voxelfieldblocks[v].solid
end


function writeFace(outfile, p, blocksize, whichface, texname)
	local points={}
	
	if whichface<1 or whichface>6 then
		-- Illegal face
		print("Illegal face ID: "..whichface)
		return
	end
	
	local fo=faceoffsets[whichface]
	local i
	for i=1,4,1 do
		points[i]={p[1]+fo[i][1]*blocksize, p[2]+fo[i][2]*blocksize, p[3]+fo[i][3]*blocksize}
	end
	
	writeBlockFace(outfile,points[1],points[2],points[3],points[4],texname)
end

function writeBlock(outfile, voxelfield, origin, p, blocksize, texname)
	local i
	-- p is the cell origin of the block
	-- Convert it to world coords
	local worldp={origin[1]+p[1]*blocksize, origin[2]+p[2]*blocksize, origin[3]+p[3]*blocksize}
	
	-- Iterate all faces and calculate the neighbor for each face
	-- Test that neighbor for being solid, and if not solid then write the face
	for i=1,6,1 do
		-- Calculate neighbor point
		local np={p[1]+blockoffsets[i][1], p[2]+blockoffsets[i][2], p[3]+blockoffsets[i][3]}
		if (testIsSolid(voxelfield,np)==false) then
			-- Neighbor does not occlude, write the face
			writeFace(outfile, worldp, blocksize, i, texname)
		end
	end
end

function writeVoxelField(outfilename, voxelfield, sx, sy, sz, origin, blocksize)
	local x,y,z
	local outfile=io.open(outfilename, "w")
	beginPOVMeshDeclaration(outfile)
	
	for x=0,sx-1,1 do
		for y=0,sy-1,1 do
			for z=0,sz-1,1 do
				local blockid=voxelfield:getVoxel(x,y,z)+1
				local texname="NONE"
				if voxelfieldblocks[blockid]==nil then
					-- Error, invalid block type
					print("Error: Invalid block type: "..blockid)
				else
					-- If block is solid and not equal to 1 then write the block
					if voxelfieldblocks[blockid].solid==true and blockid~= 1 then
						texname=voxelfieldblocks[blockid].texname
						writeBlock(outfile,voxelfield,origin,{x,y,z},blocksize,texname)
					end
				end
			end
		end
	end
	closePOVMeshDeclaration(outfile)
	outfile:close()
end


	